-- upvalue global constants for faster look up ------------
local _G = getfenv(0)
local string_match = _G.string.match
local string_split = _G.string.split
local string_format = _G.string.format
local string_sub = _G.string.sub
local string_trim = _G.string.trim
local to_number = _G.tonumber
local currResolution = _G.GetCurrentResolution
local getResolutions = _G.GetScreenResolutions
local f_mod = _G.math.fmod
local math_floor = _G.math.floor
local table_insert = _G.table.insert
local table_wipe = _G.wipe
local select = _G.select

local lib = LibStub:NewLibrary("LibUtilities-1.0", "$Revision: 1$")

if not lib then
	return -- no upgrade necessary
end

-- embedded functions beginning of file code
lib.embeds = lib.embeds or {}
do
	local mixins = {
		"PixelPerfect",
		"VisualData",
		"NumberToHex",
		"DecodeItemString",
		"Round",
		"TitleCase"
	}

	--- Embeds lib on target AddOn
	-- So you can call LibStub("LibUtilities-1.0").Embed(myAddOn)
	-- @param target AddOn in which to embed
	function lib:Embed(target)
		for k, v in pairs(mixins) do
			target[v] = self[v]
		end
		self.embeds[target] = true
		return target
	end
end -- end of do

--- Rounds a number to n decimal places
-- @paramsig number[, decimal]
-- @param number That you want rounded
-- @param decimal The number of decimal places
-- decimal defaults to 0 places
-- @return Your newly rounded number
-- @usage local myNumber = 123.456
-- myNumber = self:Round(myNumber, 2)
-- print(myNumber)
-- -- 123.46
function lib:Round(number, decimal)
	local mult = 10^(decimal or 0)
	return math_floor(number * mult + 0.5) / mult
end

-- local function, not external API -------------
local function titleCase(first, rest)
	return first:upper() .. rest:lower()
end

--- Creates Title Case for strings
-- @param str The string you want Title Cased
-- @return str now fixed
-- @usage local myString = "obi-wan kenobi"
-- myString = self:TitleCase(myString)
-- print(myString)
-- -- Obi-Wan Kenobi
function lib:TitleCase(str)
	str = string.gsub(str, "(%a)([%w_']*)", titleCase)
	return str
end

--- Scales an element to pixel perfection
-- @param number The height, width, or scale you want adjusted
-- @usage myFrame:SetHeight(myAddOn:PixelPerfect(100))
-- @usage FontInstance:SetFont("fileName", myAddOn:PixelPerfect(12), "OUTLINE")
function lib:PixelPerfect(number)
	local uiScale = lib:VisualData()
	return uiScale * number
end

--- Useful visual data
-- Probably good to call if user changes screen resolutions
-- @usage local uiScale, resolutionX, resolutionY = myAddOn:VisualData()
-- @usage UIParent:SetScale(uiScale)
-- @return uiScale Number. The corrected UI scale (768/resolutionY)
-- @return resolutionX Number. Your screen's X dimension. IE// 1920
-- @return resolutionY Number. Your screen's Y dimension. IE// 1080
function lib:VisualData()
	local uiScale, resolutionX, resolutionY = 1, 800, 600
	local resolutionNow = resolutionNow or {}
	table_wipe(resolutionNow)
	resolutionNow = ({getResolutions()})[currResolution()]
	resolutionX, resolutionY = string_split("x", resolutionNow, 2)
	resolutionX, resolutionY = to_number(resolutionX), to_number(resolutionY)
	uiScale = 768/resolutionY
	return uiScale, resolutionX, resolutionY
end

--- Utility function to break down returns
-- For an item string like any other API
-- @param itemString What you want parsed
-- @usage local bonusTable = select(15, myAddOn:DecodeItemString(itemString))
-- for i = 1, #bonusTable do
--     Print(i)
-- end
-- @return itemType String. "item", "spell", etc
-- @return itemID Number suitable for searching Wowhead
-- @return enchantID Number. Permanent enchant applied to an item
-- See http://wow.gamepedia.com/EnchantId for a list
-- @return gemID1 Embedded gem, if socketed. Re-uses enchantID indicies
-- @return gemID2 Embedded gem, if socketed. Re-uses enchantID indicies
-- @return gemID3 Embedded gem, if socketed. Re-uses enchantID indicies
-- @return gemID4 Embedded gem, if socketed. Re-uses enchantID indicies
-- @return suffixID Number. Random enchantment ID. May be negative
-- See http://wow.gamepedia.com/SuffixId for a list
-- @return uniqueID Number. Data pertaining to a specific instance of the item
-- @return linkLevel Number. Level of the character supplying the link
-- @return specializationID Number. Specialization ID of the character supplying the link
-- See GetInspectSpecialization API for details
-- @return upgradeTypeID Number. Used with upgradeID
-- @return instanceDifficultyID Number. Dungeon or raid difficultyID item was obtained
-- @return numBonuses Number. The number of bonusIDs
-- @return bonusIDs Table. A list of the bonuses by number or random suffix
-- @return upgradeID Number.
function lib:DecodeItemString(itemString)
	local bonusIDs = bonusIDs or {}
	table_wipe(bonusIDs)
	
	local itemType, itemID, enchantID, gemID1, gemID2, gemID3, gemID4, suffixID, uniqueID, linkLevel, specializationID, upgradeTypeID, instanceDifficultyID, numBonuses, affixes = string_split(":", itemString, 15)
	
	for i = 1, numBonuses do
		local bonusID = select(i, string_split(":", affixes))
		table_insert(bonusIDs, bonusID..i)
	end
	
	local upgradeID = select(numBonuses + 1, string_split(":", affixes))
	
	return itemType, itemID, enchantID, gemID1, gemID2, gemID3, gemID4, suffixID, uniqueID, linkLevel, specializationID, upgradeTypeID, instanceDifficultyID, numBonuses, bonusIDs, upgradeID
end

--- Converts a number into hexidecimal
-- @param number Number. Like 1234
-- @return hexString String. Like "04d2"
function lib:NumberToHex(number)
	local hexString = "0123456789abcdef"
	local s = ""
	while number > 0 do
		local mod = f_mod(number, 16)
		s = string_sub(hexString, mod + 1, mod + 1) .. s
		number = math_floor(number / 16)
	end
	if s == "" then s = "0" end
	return s
end

for AddOn in pairs(lib.embeds) do
	lib:Embed(AddOn)
end